
/*
 * computestats.cpp  
 *  [n,y] = computestats(Y',ndx,nQ_r);
 */

#include "mex.h"


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
    
    double *inY, *inNdx, *inNQ;
    size_t ndim, npts;                   /* size of matrix */
    double *outNcount, *outYavg;         /* output matrix */
    /* check for proper number of arguments */
    if(nrhs!=3) {
        mexErrMsgIdAndTxt("MyToolbox:computestats:nrhs","Three inputs required.");
    }
    if(nlhs!=2) {
        mexErrMsgIdAndTxt("MyToolbox:computestats:nlhs","Two outputs required.");
    }

    
        /* create a pointer to the real data in the input matrix  */
    inY = mxGetPr(prhs[0]);

    /* get dimensions of the input matrix */
    ndim = mxGetM(prhs[0]);
    npts = mxGetN(prhs[0]);

    inNdx = mxGetPr(prhs[1]);

    inNQ = mxGetPr(prhs[2]);

    int nqr = (int) inNQ[0];

// std::cout<<nqr<<std::endl;   
    /* create the output matrix */
    plhs[0] = mxCreateDoubleMatrix(nqr,1,mxREAL);
    plhs[1] = mxCreateDoubleMatrix(ndim,nqr,mxREAL);
    /* get a pointer to the real data in the output matrix */
    outNcount = mxGetPr(plhs[0]);
    outYavg = mxGetPr(plhs[1]);


    int ypr;
    int updndx;
    for(int i=0; i<npts; i++) {
       updndx = (int) inNdx[i]; // base 1

       outNcount[updndx-1]++;

       ypr = (updndx-1)*ndim;
       for(int j=0; j<ndim; j++) {
          outYavg[ypr+j] += inY[i*ndim+j];
       }
        
    }

    double div;
    for(int i=0; i<nqr; i++) {
      div = outNcount[i];
      for(int j=0; j<ndim; j++) {
         outYavg[i*ndim+j] /= div;
      }
    }
}



