
/*
 * ff_traversal_mex.cpp  
 */

#include "mex.h"
#include <limits>
#include <math.h>


/* The computational routine */
void fftraverse(double *ndx, double *dist, double *data, mwSize nrows, mwSize ncols);
double computedist(int i_base1, int j_base1, double* data, mwSize ndim, mwSize npts);

/* The gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
    
    double *inMatrix;               /* 1xN input matrix */
    size_t ndim, npts;                   /* size of matrix */
    double *outNdx, *outDist;              /* output matrix */
    /* check for proper number of arguments */
    if(nrhs!=1) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:nrhs","One inputs required.");
    }
    if(nlhs!=2) {
        mexErrMsgIdAndTxt("MyToolbox:arrayProduct:nlhs","Two outputs required.");
    }

    
        /* create a pointer to the real data in the input matrix  */
    inMatrix = mxGetPr(prhs[0]);

    /* get dimensions of the input matrix */
    ndim = mxGetM(prhs[0]);
    npts = mxGetN(prhs[0]);

    /* create the output matrix */
    plhs[0] = mxCreateDoubleMatrix((mwSize)npts,1,mxREAL);
    plhs[1] = mxCreateDoubleMatrix((mwSize)npts,1,mxREAL);

    /* get a pointer to the real data in the output matrix */
    outNdx = mxGetPr(plhs[0]);
    outDist = mxGetPr(plhs[1]);

    /* call the computational routine */
    fftraverse(outNdx,outDist, inMatrix, ndim, npts);
}


void fftraverse(double *ndx, double *dist, double *data, mwSize ndim, mwSize npts)
{
    
    double inf = std::numeric_limits<double>::infinity();
    mwSize i,j,k;

    int *sel = (int*) calloc(npts+1, sizeof(int));
    double *mindist2in = (double*) calloc(npts+1, sizeof(double));
    
    for(k=0; k<=npts; k++) mindist2in[k] = inf;
    
    
    i = 0; // next to populate
    ndx[i] = 1;
    sel[(int)ndx[i]] = 1;
    i++;        
    
    double maxmindist, curr_maxmindist, mindist, curr_mindist;
    int    maxminndx,  curr_maxminndx,  minndx,  curr_minndx;
    int prev_sel;
    
    
    while(i<npts) {
        prev_sel = (int) ndx[i-1];  // base_1
        
        maxmindist = 0;
        // compute distances to prev selection
        for(k=1;k<=npts;k++) { // k == traverse over unselected ndx via sel
            if(sel[k]==1) continue;
            curr_mindist = computedist(prev_sel, k, data, ndim, npts);
            if(curr_mindist<mindist2in[k]) {
                mindist2in[k] = curr_mindist;
            }
            curr_maxmindist = mindist2in[k];
            if(curr_maxmindist > maxmindist) {
                maxmindist = curr_maxmindist;
                maxminndx = k;
            }
        }
        dist[i-1] = maxmindist;
        ndx[i] = maxminndx;
        sel[maxminndx] = 1;
        i++;
    }
    dist[npts] = 0.0;
}
        

double computedist(int i_base1, int j_base1, double* data, mwSize ndim, mwSize npts) {
    int i=i_base1-1;
    int j=j_base1-1;
    
    double dist = 0.0;
    double diff;
    
    for(int k=0;k<ndim;k++) {
        diff = data[i*ndim+k] - data[j*ndim+k];
        dist+= (diff*diff);
    }
    
    return sqrt(dist);
}