
%% load data

rng(23); % fixing the random seed for consistent results

load('example_dataset/sarcos_dataset.mat');
xdata = sarcos_inv_x;
ydata = sarcos_inv_y(:,1);
dataset_name = 'SARCOS robotic arm';
[npts, ndim] = size(xdata);

prm = randperm(size(xdata,1));
testx = xdata(prm(1:2000),:);
testy = ydata(prm(1:2000),:);
trainx = xdata(prm(2001:end),:);
trainy = ydata(prm(2001:end),:);

kernel_bandwidth = 7.5;  % this should be set using cross-validation
rnets_bandwidth = 7.5;   % this should be set using cross-validation

%% vanilla kernel prediction

fprintf('kernel training...');
kernel_learned_params = kernel_regressor_train(trainx, trainy);
fprintf('done.\n');

fprintf('kernel testing...');
tic;
kernel_ytest_pred = kernel_regressor_predict(testx, kernel_learned_params, kernel_bandwidth, @kernel_triangle);
kernel_eval_time = toc;
fprintf('done.\n');

kernel_loss = rmse_loss(kernel_ytest_pred,testy);


%% rnets prediction
fprintf('building farthest first traveral structure for rnets (this preprocessing takes about 30sec to run)...');
ff_data = [];
[ff_data.Qall, ff_data.Qdst, ff_data.Qndx] = ff_traversal(trainx);
fprintf('done.\n');

alpha_range = [1/6 2/6 3/6 4/6 5/6 6/6];
rnets_eval_time_peralpha = nan(size(alpha_range));
rnets_loss_peralpha = nan(size(alpha_range));
for alphai = 1:length(alpha_range)
    alpha = alpha_range(alphai);
    fprintf('rnets training (alpha = %1.2f)...',alpha);
    rnets_learned_params = rnets_train(trainx,trainy, alpha, rnets_bandwidth, ff_data);
    fprintf('done.\n');

    fprintf('rnets testing...');
    tic;
    rnet_ytest_pred = rnets_predict(testx, rnets_learned_params, @kernel_triangle);
    rnets_eval_time_peralpha(alphai) = toc;
    fprintf('done.\n');
    rnets_loss_peralpha(alphai) = rmse_loss(rnet_ytest_pred,testy);
end


%% comparitive statistics

fprintf('\n\n  --- comparitive statistics --- \n\n');
fprintf('\nDataset name: %s (pts = %d, dim = %d)\n\n', dataset_name, npts, ndim);
fprintf('\talpha\taccuracy\tspeedup\n');
for alphai = 1:length(alpha_range)
    alpha = alpha_range(alphai);
    fprintf('\t%1.2f\t  %1.2f\t\t  %1.2f\n',alpha, kernel_loss/rnets_loss_peralpha(alphai), kernel_eval_time/rnets_eval_time_peralpha(alphai));
end
fprintf('\n\n');
fprintf('  accuracy and speedup columns show the prediction quality and prediction\n');
fprintf('  speed of rnets compared to kernel regression (higher is better). A value\n');
fprintf('  of 1.00 means same performance as kernels, 2.00 means twice better, etc.\n');
fprintf('\n\n');


